#include <allegro.h>
#include <stdlib.h>

#include "common.h"
#include "region.h"

int main(int argc, char *argv[])
 {
  DirtyRectangles rects;
  DRInit(&rects);
  
  allegro_init();
  install_mouse();
  install_keyboard();
  
  if(set_gfx_mode(GFX_AUTODETECT_WINDOWED, 640, 480, 0, 0))
   return EXIT_FAILURE;
  
  #ifdef OLD_ALLEGRO
  text_mode(-1);
  #endif
  
  BITMAP *buffer = create_bitmap(SCREEN_W, SCREEN_H);
  if(!buffer)
   return EXIT_FAILURE;
  
  int drawing = 0;
  Region region = {0, 0, 0, 0}, temp = {0, 0, 0, 0};
  
  while(!key[KEY_ESC])
   {
    temp = region;
    
    if(!regionIsNormal(&temp))
     regionNormalize(&temp);
    
    if(mouse_b & 1)
     {
      if(!drawing)
       {
        drawing = 1;
        region.x1 = mouse_x;
        region.y1 = mouse_y;
       }
      
      region.x2 = mouse_x;
      region.y2 = mouse_y;
     }
    else
     {
      if(drawing)
       {
        if(key[KEY_LSHIFT])
         DRRemove(&rects, &temp);
        else if(key[KEY_LCONTROL])
         DRClip(&rects, &temp);
        else
         DRAdd(&rects, &temp);
        
        drawing = 0;
       }
     }
    
    if(!DRIsSaturated(&rects))
     {
      clear(buffer);
      
      Region *r = rects.start;
      
      for(; r != rects.end; ++r)
       {
        if(drawing && regionCollision(r, &temp))
         rectfill(buffer, r->x1+1, r->y1+1, r->x2-2, r->y2-2, makecol(128, 0, 0));
        else
         rectfill(buffer, r->x1+1, r->y1+1, r->x2-2, r->y2-2, makecol(0, 0, 128));
       }
      
      for(r = rects.start; r != rects.end; ++r)
       if(drawing && regionCollision(r, &temp))
        rect(buffer, r->x1, r->y1, r->x2-1, r->y2-1, makecol(255, 0, 0));
       else
        rect(buffer, r->x1, r->y1, r->x2-1, r->y2-1, makecol(0, 0, 255));
      
      for(r = rects.start; r != rects.end; ++r)
       #ifdef OLD_ALLEGRO
       textprintf_centre(buffer, font, (r->x1+r->x2)/2, (r->y1+r->y2)/2-3, makecol(255, 255, 255), "%d", r-rects.start);
       #else
       textprintf_centre_ex(buffer, font, (r->x1+r->x2)/2, (r->y1+r->y2)/2-3, makecol(255, 255, 255), -1, "%d", r-rects.start);
       #endif
     }
    else
     {
      clear_to_color(buffer, makecol(255, 128, 0));
      #ifdef OLD_ALLEGRO
      textout_centre(buffer, font, "Buffer Saturated.", 320, 240, makecol(255, 255, 255));
      #else
      textout_centre_ex(buffer, font, "Buffer Saturated.", 320, 240, makecol(255, 255, 255), -1);
      #endif
     }
    
    if(drawing)
     {
      if(key[KEY_LSHIFT])
       rect(buffer, temp.x1, temp.y1, temp.x2-1, temp.y2-1, makecol(255, 255, 0));
      else if(key[KEY_LCONTROL])
       rect(buffer, temp.x1, temp.y1, temp.x2-1, temp.y2-1, makecol(255, 0, 0));
      else
       rect(buffer, temp.x1, temp.y1, temp.x2-1, temp.y2-1, makecol(0, 255, 0));
     }
    
    if(key[KEY_O])
     DROptimize(&rects);
    
    vline(buffer, mouse_x, mouse_y-5, mouse_y-2, makecol(255, 255, 255));
    vline(buffer, mouse_x, mouse_y+2, mouse_y+5, makecol(255, 255, 255));
    hline(buffer, mouse_x-5, mouse_y, mouse_x-2, makecol(255, 255, 255));
    hline(buffer, mouse_x+2, mouse_y, mouse_x+5, makecol(255, 255, 255));
    
    #ifdef OLD_ALLEGRO
    textout(buffer, font, "Dirty Rectangle Test:", 0, 0, makecol(192, 192, 192));
    textout(buffer, font, "Click and drag to select region to add region to list.", 0, 8, makecol(192, 192, 192));
    textout(buffer, font, "Hold left shift to remove that region.", 0, 16, makecol(192, 192, 192));
    textout(buffer, font, "Hold left ctrl to clip to that region.", 0, 24, makecol(192, 192, 192));
    textout(buffer, font, "Press O to optimize the rectangles.", 0, 32, makecol(192, 192, 192));
    textout(buffer, font, "ESC to quit.", 0, 40, makecol(192, 192, 192));
    #else
    textout_ex(buffer, font, "Dirty Rectangle Test:", 0, 0, makecol(192, 192, 192), -1);
    textout_ex(buffer, font, "Click and drag to select region to add region to list.", 0, 8, makecol(192, 192, 192), -1);
    textout_ex(buffer, font, "Hold left shift to remove that region.", 0, 16, makecol(192, 192, 192), -1);
    textout_ex(buffer, font, "Hold left ctrl to clip to that region.", 0, 24, makecol(192, 192, 192), -1);
    textout_ex(buffer, font, "Press O to optimize the rectangles.", 0, 32, makecol(192, 192, 192), -1);
    textout_ex(buffer, font, "ESC to quit.", 0, 40, makecol(192, 192, 192), -1);
    #endif
    
    blit(buffer, screen, 0, 0, 0, 0, SCREEN_W, SCREEN_H);
   }
  
  allegro_exit();
  return EXIT_SUCCESS;
 } END_OF_MAIN();
