#include "common.h"
#include "gui.h"
#include "config.h"

#include <allegro.h>
#include <stdlib.h>

typedef struct
 {
  int changed;
  
  struct
   {
    int x, y;
   } p[8];
  
  int key;
  
 } UpdateData;

int UpdateTestWindow(Window *w)
 {
  assert(w);
  assert(w->data);
  
  UpdateData *data = w->data;
  
  if(data->changed)
   {
    data->changed = 0;
    w->attr |= attr_dirty;
    GUIMarkDirtyAll(w);
   }
  
  return 1;
 }

void RedrawTestWindow(Window *w)
 {
  UpdateData *data = w->data;
  clear_to_color(w->buffer, (int)w);
  
  int c[8] = {makecol32(255, 255, 255), makecol32(255, 0, 0), makecol32(0, 255, 0), makecol32(0, 0, 255),  makecol32(128, 128, 128), makecol32(128, 0, 0), makecol32(0, 128, 0), makecol32(0, 0, 128)};
  typedef void(*circlefunc)(BITMAP *,int,int,int,int);
  circlefunc f[8] = {&circlefill, &circlefill, &circlefill, &circlefill, &circle, &circle, &circle, &circle};
  
  int i = 0;
  for(; i < 8; ++i)
   (*(f[i]))(w->buffer, data->p[i].x, data->p[i].y, 1+i, c[i]);
  
  #ifdef OLD_ALLEGRO
  textprintf_centre(w->buffer, font, w->buffer->w/2, w->buffer->h/2, makecol32(255, 255, 255), "%c", data->key);
  #else
  textprintf_centre_ex(w->buffer, font, w->buffer->w/2, w->buffer->h/2, makecol32(255, 255, 255), 0, "%c", data->key);
  #endif
 }

int KeyTestWindow(Window *w, int value, int scancode)
 {
  UpdateData *data = w->data;
  data->key = value;
  data->changed = 1;
  return 1;
 }

int MouseTestWindow(Window *w, int x, int y, int xoff, int yoff, int b, int b_off)
 {
  assert(w);
  assert(w->data);
  
  if(b&1)
   GUIMoveWindow(w, xoff, yoff);
  
  UpdateData *data = w->data;
  data->p[0].x = x;
  data->p[0].y = y;
  
  if(b&1)
   {
    data->p[1].x = x;
    data->p[1].y = y;
   }
  
  if(b&2)
   {
    data->p[2].x = x;
    data->p[2].y = y;
   }
  
  if(b&4)
   {
    data->p[3].x = x;
    data->p[3].y = y;
   }
  
  data->p[4].x = x-xoff;
  data->p[4].y = y-yoff;
  
  if(b_off&1)
   {
    data->p[5].x = x;
    data->p[5].y = y;
   }
  
  if(b_off&2)
   {
    data->p[6].x = x;
    data->p[6].y = y;
   }
  
  if(b_off&4)
   {
    data->p[7].x = x;
    data->p[7].y = y;
   }
  
  data->changed = 1;
  return 1;
 }

int TestWindow(Window *w, void *unused)
 {
  w->update = &UpdateTestWindow;
  w->mouse = &MouseTestWindow;
  w->redraw = &RedrawTestWindow;
  w->key = &KeyTestWindow;
  
  w->attr |= attr_select;
  
  if(w->buffer->w > 40 && w->buffer->h > 40)
   {
    int width = (w->buffer->w - 20)/2, height = (w->buffer->h - 20)/2;
    GUIAddChild(w, GUICreateWindow(&TestWindow, 0, 5, 5, width, height));
    GUIAddChild(w, GUICreateWindow(&TestWindow, 0, w->buffer->w-5-width, 5, width, height));
    GUIAddChild(w, GUICreateWindow(&TestWindow, 0, 5, w->buffer->h-5-height, width, height));
    GUIAddChild(w, GUICreateWindow(&TestWindow, 0, w->buffer->w-5-width, w->buffer->h-5-height, width, height));
   }
  
  if(!(w->data = malloc(sizeof(UpdateData))))
   return 0;
  
  UpdateData *data = w->data;
  data->changed = 1;
  
  return 1;
 }

int main(int argc, char *argv[])
 {
  if(allegro_init())
   return EXIT_FAILURE;
  
  install_keyboard();
  install_mouse();
  install_timer();
  
  GUIInit(640, 480, 32, 0);
  
  GUIAddRoot(GUICreateWindow(&BaseWindow, 0, 50, 50, 200, 200));
  GUIAddRoot(GUICreateWindow(&TestWindow, 0, 10, 10, 400, 400));
  
  while(!key[KEY_ESC])
   {
    poll_keyboard();
    GUIUpdate();
   }
  
  GUIUninit();
  
  allegro_exit();
  return EXIT_SUCCESS;
 } END_OF_MAIN();
