#include "common.h"
#include "log.h"
#include <stdarg.h>
#include <stdio.h>
#include <string.h>

static FILE *logfile = 0;

#ifdef CLIENT
    #define LOGFILE "client.log"
#endif

#ifdef SERVER
#ifdef LOGFILE
#undef LOGFILE
#define LOGFILE "log.log"
#else
#define LOGFILE "server.log"
#endif
#endif

#ifndef LOGFILE
#define LOGFILE "log.log"
#endif

static const char *logTypeString(enum LogType type)
 {
  switch(type)
   {
    case Debug:
     return "DEBUG";
    
    case Notice:
     return "NOTICE";
    
    case Warning:
     return "WARNING";
    
    case Error:
     return "ERROR";
    
    default:
     return "UNKNOWN";
   }
 }

void logStop(void)
 {
  if(logfile)
   {
    fprintf(logfile, "%s [%s] %s\n", dateString(), logTypeString(Notice), "Logging Stopped.");
    fclose(logfile);
    logfile = 0;
   }
 }

int logStart()
 {
  if(!logfile)
   {
    if(!(logfile = fopen(LOGFILE, "a")))
     return 0;
    
    fprintf(logfile, "%s [%s] %s\n", dateString(), logTypeString(Notice), "Logging Started.");
    fflush(logfile);
   }
  
  return 1;
 }

static void vLogMessage(enum LogType type, const char *string, va_list args)
 {
  static char buffer[1024];
  
  #ifdef NDEBUG
  if(type == Debug)
   return;
  #endif
  
  vsnprintf(buffer, sizeof buffer, string, args);
  
  if(!logfile)
   {
    if(logStart())
     atexit(&logStop);
    else
     {
      fprintf(stderr, "Unable to open " LOGFILE ", missing entry was: %s", buffer);
      return;
     }
   }
  
  fprintf(logfile, "%s [%s] %s\n", dateString(), logTypeString(type), buffer);
  fflush(logfile);
  
  fprintf(stderr, "[%s] %s\n", logTypeString(type), buffer);
  fflush(stderr);
 }

void logMessage(enum LogType type, const char *string, ...)
 {
  va_list args;
  va_start(args, string);
  vLogMessage(type, string, args);
  va_end(args);
 }
